*****************************************************************************
*** FUNCTION MFphih3 calculates the flux above threshold (flxk=1) or the
*** differntial flux (flxk=2) from a given Higgs
*** boson decaying in flight, the energy of the Higgs boson should be given
*** in EH.
*** Higgses HNO = 1-4 are supported (H10, H20, H30 and H+/H-)
*** Units: 1.0E-30 m**-2 (annihilation)**-1
*****************************************************************************

      real*8 function MFphih3(eh,emuth,thmax,hno,wh,flxk,istat)
      implicit none
      include 'MFCom.h'
      
C------------------------ Variables ------------------------------------

      real*8 eh,emuth,thmax
      integer hno,istat,flxk
      character*2 wh

      real*8 e1,e2,flx
      integer dch

C------------------------ Functions ------------------------------------

      real*8 MFphih4,MFemean,MFphiith

C-----------------------------------------------------------------------


C...Loop through different decay channels
      flx=0.0d0

C---------- Neutral Higgs bosons ----------
      if (hno.le.3) then

c..."fundamental" channels
        do 100 dch=1,6
          if (hdw(dch,hno).gt.0.0) then
            flx=flx+hdw(dch,hno)*2.0d0*
     &        MFphiith(eh,mh(hno),map(dch),map(dch),emuth,thmax,
     &          dch,wh,flxk,istat)
          endif
  100   continue

c..."complex" channels
        if (hdw(7,hno).gt.0.0d0) then    ! H10 H10 channel
          e1=MFemean(eh,mh(hno),mh(1),mh(1))
          flx=flx+2.0*hdw(7,hno)*MFphih4(e1,emuth,thmax,1,wh,flxk,istat)
        endif

        if (hdw(8,hno).gt.0.0d0) then    ! H20 H20 channel
          e1=MFemean(eh,mh(hno),mh(2),mh(2))
          flx=flx+2.0*hdw(8,hno)*MFphih4(e1,emuth,thmax,2,wh,flxk,istat)
        endif

        if (hdw(9,hno).gt.0.0d0) then    ! H30 H30 channel
          e1=MFemean(eh,mh(hno),mh(3),mh(3))
          flx=flx+2.0*hdw(9,hno)*MFphih4(e1,emuth,thmax,3,wh,flxk,istat)
        endif

        if (hdw(10,hno).gt.0.0d0) then    ! H+ H- channel
          e1=MFemean(eh,mh(hno),mh(4),mh(4))
          flx=flx+2.0*
     &      hdw(10,hno)*MFphih4(e1,emuth,thmax,4,wh,flxk,istat)
        endif
         
        if (hdw(11,hno).gt.0.0d0) then    ! Z0 H10 channel
          e2=MFemean(eh,mh(hno),mh(1),map(6))
          flx=flx+hdw(11,hno)*
     +    MFphiith(eh,mh(hno),map(6),mh(1),emuth,thmax,6,wh,flxk,istat)
          flx=flx+hdw(11,hno)*MFphih4(e2,emuth,thmax,1,wh,flxk,istat)
        endif
         
        if (hdw(12,hno).gt.0.0d0) then    ! Z0 H20 channel
          e2=MFemean(eh,mh(hno),mh(2),map(6))
          flx=flx+hdw(12,hno)*
     +    MFphiith(eh,mh(hno),map(6),mh(2),emuth,thmax,6,wh,flxk,istat)
          flx=flx+hdw(12,hno)*MFphih4(e2,emuth,thmax,2,wh,flxk,istat)
        endif
         
        if (hdw(13,hno).gt.0.0d0) then    ! Z0 H30 channel
          e2=MFemean(eh,mh(hno),mh(3),map(6))
          flx=flx+hdw(13,hno)*
     +    MFphiith(eh,mh(hno),map(6),mh(3),emuth,thmax,6,wh,flxk,istat)
          flx=flx+hdw(13,hno)*MFphih4(e2,emuth,thmax,3,wh,flxk,istat)
        endif
         
        if (hdw(14,hno).gt.0.0d0) then    ! W-H+ W+H- channel
          e2=MFemean(eh,mh(hno),mh(4),map(5))
          flx=flx+hdw(14,hno)*
     +    MFphiith(eh,mh(hno),map(5),mh(4),emuth,thmax,5,wh,flxk,istat)
          flx=flx+hdw(14,hno)*MFphih4(e2,emuth,thmax,4,wh,flxk,istat)
        endif

C---------- Charged Higgs bosons ----------
      else

        if (hdw(3,hno).gt.0.0d0) then ! u b-bar
          flx=flx+hdw(3,hno)*
     &    MFphiith(eh,mh(hno),map(2),mqu,emuth,thmax,2,wh,flxk,istat)
        endif

        if (hdw(4,hno).gt.0.0d0) then ! c d-bar
          flx=flx+hdw(4,hno)*
     &    MFphiith(eh,mh(hno),map(1),mqd,emuth,thmax,1,wh,flxk,istat)
        endif

        if (hdw(5,hno).gt.0.0d0) then ! c s-bar
          flx=flx+hdw(5,hno)*
     &    MFphiith(eh,mh(hno),map(1),mqs,emuth,thmax,1,wh,flxk,istat)
        endif

        if (hdw(6,hno).gt.0.0d0) then ! c b-bar
          flx=flx+hdw(6,hno)*
     &    MFphiith(eh,mh(hno),map(1),map(2),emuth,thmax,
     &    1,wh,flxk,istat)
          flx=flx+hdw(6,hno)*
     &    MFphiith(eh,mh(hno),map(2),map(1),emuth,thmax,
     &    2,wh,flxk,istat)
        endif

        if (hdw(7,hno).gt.0.0d0) then ! t d-bar
          flx=flx+hdw(7,hno)*
     &    MFphiith(eh,mh(hno),map(3),mqd,emuth,thmax,
     &    3,wh,flxk,istat)
        endif

        if (hdw(8,hno).gt.0.0d0) then ! t s-bar
          flx=flx+hdw(8,hno)*
     &    MFphiith(eh,mh(hno),map(3),mqs,emuth,thmax,
     &    3,wh,flxk,istat)
        endif

        if (hdw(9,hno).gt.0.0d0) then ! t b-bar
          flx=flx+hdw(9,hno)*
     &    MFphiith(eh,mh(hno),map(3),map(2),emuth,thmax,
     &    3,wh,flxk,istat)
          flx=flx+hdw(9,hno)*
     &    MFphiith(eh,mh(hno),map(2),map(3),emuth,thmax,
     &    2,wh,flxk,istat)
        endif

        if (hdw(12,hno).gt.0.0d0) then ! tau nu_tau
          flx=flx+hdw(12,hno)*
     &    MFphiith(eh,mh(hno),map(4),0.0d0,emuth,thmax,
     &    4,wh,flxk,istat)
        endif

        if (hdw(13,hno).gt.0.0d0) then ! W+ H1
          flx=flx+hdw(13,hno)*
     &    MFphiith(eh,mh(hno),map(5),mh(1),emuth,thmax,
     &    5,wh,flxk,istat)
          e2=MFemean(eh,mh(hno),mh(1),map(5))
          flx=flx+hdw(13,hno)*
     &      MFphih4(e2,emuth,thmax,1,wh,flxk,istat)
        endif

        if (hdw(14,hno).gt.0.0d0) then ! W+ H2
          flx=flx+hdw(14,hno)*
     &    MFphiith(eh,mh(hno),map(5),mh(2),emuth,thmax,
     &    5,wh,flxk,istat)
          e2=MFemean(eh,mh(hno),mh(2),map(5))
          flx=flx+hdw(14,hno)*
     &      MFphih4(e2,emuth,thmax,2,wh,flxk,istat)
        endif

        if (hdw(15,hno).gt.0.0d0) then ! W+ H3
          flx=flx+hdw(15,hno)*
     &    MFphiith(eh,mh(hno),map(5),mh(3),emuth,thmax,
     &    5,wh,flxk,istat)
          e2=MFemean(eh,mh(hno),mh(3),map(5))
          flx=flx+hdw(15,hno)*
     &      MFphih4(e2,emuth,thmax,3,wh,flxk,istat)
        endif

      endif

      MFphih3 = flx

      end


