*****************************************************************************
*** FUNCTION MFphih4 calculates the flux above threshold (flxk=1) or the
*** differential flux (flxk=2) from a given Higgs
*** boson decaying in flight, the energy of the Higgs boson should be given
*** in EH.
*** Higgses HNO = 1-4 are supported (H10, H20, H30 and H+/H-)
*** Units: 1.0E-30 m**-2 (annihilation)**-1
*****************************************************************************

      real*8 function MFphih4(eh,emuth,thmax,hno,wh,flxk,istat)
      implicit none
      include 'MFCom.h'

C------------------------ Variables ------------------------------------

      real*8 eh,emuth,thmax
      integer hno,istat,flxk
      character*2 wh

      real*8 flx
      integer dch

C------------------------ Functions ------------------------------------

      real*8 MFphiith

C-----------------------------------------------------------------------


C...Loop through different decay channels
      flx=0.0d0

C---------- Neutral Higgs bosons ----------
      if (hno.le.3) then

c..."fundamental" channels
        do 100 dch=1,6
          if (hdw(dch,hno).gt.0.0d0) then
            flx=flx+hdw(dch,hno)*2.0d0*
     &        MFphiith(eh,mh(hno),map(dch),map(dch),emuth,thmax,
     &          dch,wh,flxk,istat)
          endif
  100   continue


C---------- Charged Higgs bosons ----------
      else

        if (hdw(3,hno).gt.0.0d0) then ! u b-bar
          flx=flx+hdw(3,hno)*
     &    MFphiith(eh,mh(hno),map(2),mqu,emuth,thmax,2,wh,flxk,istat)
        endif

        if (hdw(4,hno).gt.0.0d0) then ! c d-bar
          flx=flx+hdw(4,hno)*
     &    MFphiith(eh,mh(hno),map(1),mqd,emuth,thmax,1,wh,flxk,istat)
        endif

        if (hdw(5,hno).gt.0.0d0) then ! c s-bar
          flx=flx+hdw(5,hno)*
     &    MFphiith(eh,mh(hno),map(1),mqs,emuth,thmax,1,wh,flxk,istat)
        endif

        if (hdw(6,hno).gt.0.0d0) then ! c b-bar
          flx=flx+hdw(6,hno)*
     &    MFphiith(eh,mh(hno),map(1),map(2),emuth,thmax,
     &    1,wh,flxk,istat)
          flx=flx+hdw(6,hno)*
     &    MFphiith(eh,mh(hno),map(2),map(1),emuth,thmax,
     &    2,wh,flxk,istat)
        endif

        if (hdw(7,hno).gt.0.0d0) then ! t d-bar
          flx=flx+hdw(7,hno)*
     &    MFphiith(eh,mh(hno),map(3),mqd,emuth,thmax,
     &    3,wh,flxk,istat)
        endif

        if (hdw(8,hno).gt.0.0d0) then ! t s-bar
          flx=flx+hdw(8,hno)*
     &    MFphiith(eh,mh(hno),map(3),mqs,emuth,thmax,
     &    3,wh,flxk,istat)
        endif

        if (hdw(9,hno).gt.0.0d0) then ! t b-bar
          flx=flx+hdw(9,hno)*
     &    MFphiith(eh,mh(hno),map(3),map(2),emuth,thmax,
     &    3,wh,flxk,istat)
          flx=flx+hdw(9,hno)*
     &    MFphiith(eh,mh(hno),map(2),map(3),emuth,thmax,
     &    2,wh,flxk,istat)
        endif

        if (hdw(12,hno).gt.0.0d0) then ! tau nu_tau
          flx=flx+hdw(12,hno)*
     &    MFphiith(eh,mh(hno),map(4),0.0d0,emuth,thmax,
     &    4,wh,flxk,istat)
        endif

      endif

      MFphih4 = flx

C...Check for inconsistent declarations in HDW
      if (hno.le.3) then ! Neutral Higgses
        do dch=7,14
          if (hdw(dch,hno).gt.0.0d0) then
          write(6,*) 'ERROR in MFphih4: inconsistent decay widths',
     +      ' declared in hdw/HDWidth.'
          write(6,*) 'Model: ',MFtag
          endif
        enddo
      else               ! Charged Higgs
        do dch=13,15
          if (hdw(dch,hno).gt.0.0d0) then
          write(6,*) 'ERROR in MFphih4: inconsistent decay widths',
     +      ' declared in hdw/HDWidth.'
          write(6,*) 'Model: ',MFtag
          endif
        enddo
      endif

      end




