*****************************************************************************
*** FUNCTION MFphii calculates the muon flux above threshold and within
*** the selected angular window (flxk=1) or the differential flux
*** (flxk=2) for channel CH.
*** Only channels CH = 1-6 are supported.
*** Units: 1.0E-30 m**-2 (annihilation)**-1  integrated
*** Units: 1.0E-30 m**-2 GeV**-1 (degree)**-1 (annihilation)**1 differential
*** Author: Joakim Edsjo, edsjo@physics.berkeley.edu
*** Date: 1995
*** Modified: Dec 03, 1997.
*****************************************************************************

      real*8 function MFphii(mneu,emuthr,thmax,ch,wh,flxk,istat)
      implicit none
      include 'MFCom.h'

C------------------------ Variables ------------------------------------

      real*8 mneu,emuthr,thmax,phi1,phi2,mp1,mp2,zpl,thpl,mn,z,th,
     &  tmp
      integer ch,istat,zi,thi,m1i,m2i,whi,flxk
      character*2 wh
      logical wb

C-----------------------------------------------------------------------

      call MFCom ! Set up common block variables

      wb=.true.
      if (wh.eq.'su'.or.wh.eq.'SU') then
        whi=1
      else
        whi=2
      endif

      if (emuthr.ge.mneu) then
        MFphii=0.0d0
        return
      endif

      mn = mneu
c...Take care of the case where mneu is between the mass of the annihilation
c...product and the lower bound of the simulations (there might be a small
c...gap of less than a GeV) for tt-bar,ww and zz.
      if (ch.eq.3.or.ch.eq.5.or.ch.eq.6) then
        if (mn.lt.lb(ch).and.mn.gt.(0.99*lb(ch))) then
          mn=lb(ch)
        endif
      endif
c...check if mneu within correct bounds
      if (mn.lt.lb(ch).and.emuthr.lt.lb(ch)) then 
        if (MFprt.gt.0) then
          write(6,*)
          write(6,5000) 'WARNING IN MFphii: A neutralino mass of ',mn,
     +      ' GeV wants to be used,'
          write(6,5010) 'while the lower bound for channel ',ch,
     +      ' is ',lb(ch),' GeV.'
          write(6,*) 'The flux is put to 0.0 for these too low masses.' 
          write(6,*) 'The results can thus only be trusted as a',
     +      ' lower bound.'
          write(6,*) 'Model: ',MFtag
        endif
        wb=.false.
        MFphii=0.0d0
        istat=(istat/2)*2+1
      endif

      if (mn.gt.ub(ch)) then 
        if (MFprt.gt.0) then
          write(6,*)
          write(6,5000) 'WARNING in MFphii: a neutralino mass of ',mn,
     +      ' GeV wants to be used,'
          write(6,5010) 'while the upper bound for channel ',ch,
     +      ' is ',ub(ch),' GeV.'
          write(6,5020) 'a neutralino mass of ',ub(ch),' GeV is used',
     +      ' instead for these too high masses.'
          write(6,*) 'The results can thus only be trusted as a',
     +      ' lower bound.'
          write(6,*) 'Model: ',MFtag
        endif
        istat=(istat/2)*2+1
      endif


c---------------------------------------------------- Integrated fluxes
      if (wb.and.flxk.eq.1) then
c...Determine which entries in MFphiint to use and how
        z=emuthr/mn
c        zi=5
        call MFifind(0,zn-1,z,zindex(0,1),zi,zpl)
c        do i=0,zn-1
c          if (z.ge.zindex(i,1).and.z.lt.zindex(i+1,1)) then
c            zi=i
c            zpl=(z-zindex(i,1))/(zindex(i+1,1)-zindex(i,1))
c            goto 21
c          endif
c        enddo
c 21     continue

        if (zi.eq.-5.or.zi.ge.zn) then
          MFphii=0.0d0
          return
        endif

c        thi=-5
        th=thmax
        call MFifind(0,thn-1,th,thindex(0,1),thi,thpl)
c        do i=0,thn-1
c          if (th.ge.thindex(i,1).and.th.lt.thindex(i+1,1)) then
c            thi=i
c            thpl=(th-thindex(i,1))/(thindex(i+1,1)-thindex(i,1))
c            goto 22
c          endif
c        enddo
c 22     continue

        if (thmax.gt.30.0d0) then
          thi=thn-1
          thpl=1.0d0
        endif
        if (thi.eq.-5) then
          MFphii=0.0d0
          return
        endif

        call MFifind(1,17,mn,mi(1),m1i,tmp)
        mp1=mi(m1i)
        m2i=m1i+1
        mp2=mi(m2i)

c        do i=1,17
c          if (mn.ge.mi(i).and.mn.lt.mi(i+1)) then
c            mp1=mi(i)
c            m1i=i
c            mp2=mi(i+1)
c            m2i=i+1
c          endif
c        enddo
        if (mn.ge.mi(18)) then
          m1i=18
          m2i=18
          mp1=mi(18)
          mp2=mp1

            MFphii = (1.0-thpl)*((1.0-zpl)*
     &      dble(phiint(thi,zi,m1i,ch,whi))+ 
     &      zpl*dble(phiint(thi,zi+1,m1i,ch,whi)))+
     &      thpl*((1.0-zpl)*dble(phiint(thi+1,zi,m1i,ch,whi))+
     &      zpl*dble(phiint(thi+1,zi+1,m1i,ch,whi)))
        else
          phi1 = (1.0-thpl)*((1.0-zpl)*
     &      dble(phiint(thi,zi,m1i,ch,whi))+ 
     &      zpl*dble(phiint(thi,zi+1,m1i,ch,whi)))+
     &      thpl*((1.0-zpl)*dble(phiint(thi+1,zi,m1i,ch,whi))+
     &      zpl*dble(phiint(thi+1,zi+1,m1i,ch,whi)))
          phi2 = (1.0-thpl)*((1.0-zpl)*
     &      dble(phiint(thi,zi,m2i,ch,whi))+ 
     &      zpl*dble(phiint(thi,zi+1,m2i,ch,whi)))+
     &      thpl*((1.0-zpl)*dble(phiint(thi+1,zi,m2i,ch,whi))+
     &      zpl*dble(phiint(thi+1,zi+1,m2i,ch,whi)))
          MFphii = phi1 + (phi2-phi1)*(mn-mp1)*(mn+mp1)/
     &      ((mp2-mp1)*(mp2+mp1))
        endif
        if (mn.ne.mneu) then
          MFphii=mneu**2/mn**2*MFphii
        endif
      endif

c-------------------------------------------------- Differential fluxes
      if (wb.and.flxk.eq.2) then
c...Determine which entries in phidiff to use and how
        z=emuthr/mn
        call MFifind(-1,zn-1,z,zindex(-1,2),zi,zpl)

        if (zi.eq.-5.or.zi.ge.zn) then
          MFphii=0.0d0
          return
        endif

c        zi=-5
c        do i=-1,zn-1
c          if (z.ge.zindex(i,2).and.z.lt.zindex(i+1,2)) then
c            zi=i
c            zpl=(z-zindex(i,2))/(zindex(i+1,2)-zindex(i,2))
c            goto 24
c          endif
c        enddo
c 24     continue
        
        if ((emuthr.gt.mn).or.(emuthr.le.0.0).or.zi.eq.-5) then
          MFphii=0.0d0
          return
        endif

c        thi=-5
        th=thmax
        call MFifind(-1,thn-1,th,thindex(-1,2),thi,thpl)

c        do i=-1,thn-1
c          if (th.ge.thindex(i,2).and.th.lt.thindex(i+1,2)) then
c            thi=i
c            thpl=(th-thindex(i,2))/(thindex(i+1,2)-thindex(i,2))
c            goto 25
c          endif
c        enddo
c 25     continue

        if (thmax.gt.30.0d0.or.thi.eq.-5) then
          MFphii=0.0d0
          return
        endif

        call MFifind(1,17,mn,mi(1),m1i,tmp)
        mp1=mi(m1i)
        m2i=m1i+1
        mp2=mi(m2i)
c        do i=1,17
c          if (mn.ge.mi(i).and.mn.lt.mi(i+1)) then
c            mp1=mi(i)
c            m1i=i
c            mp2=mi(i+1)
c            m2i=i+1
c          endif
c        enddo
        if (mn.ge.mi(18)) then
          m1i=18
          m2i=18
          mp1=mi(18)
          mp2=mp1
          MFphii = (1.0-thpl)*((1.0-zpl)*
     &      dble(phidiff(thi,zi,m1i,ch,whi))+ 
     &      zpl*dble(phidiff(thi,zi+1,m1i,ch,whi)))+
     &      thpl*((1.0-zpl)*dble(phidiff(thi+1,zi,m1i,ch,whi))+
     &      zpl*dble(phidiff(thi+1,zi+1,m1i,ch,whi)))
        else
          phi1 = (1.0-thpl)*((1.0-zpl)*
     &      dble(phidiff(thi,zi,m1i,ch,whi))+ 
     &      zpl*dble(phidiff(thi,zi+1,m1i,ch,whi)))+
     &      thpl*((1.0-zpl)*dble(phidiff(thi+1,zi,m1i,ch,whi))+
     &      zpl*dble(phidiff(thi+1,zi+1,m1i,ch,whi)))
          phi2 = (1.0-thpl)*((1.0-zpl)*
     &      dble(phidiff(thi,zi,m2i,ch,whi))+ 
     &      zpl*dble(phidiff(thi,zi+1,m2i,ch,whi)))+
     &      thpl*((1.0-zpl)*dble(phidiff(thi+1,zi,m2i,ch,whi))+
     &      zpl*dble(phidiff(thi+1,zi+1,m2i,ch,whi)))
          MFphii = phi1 + (phi2-phi1)*(mn-mp1)*(mn+mp1)/
     &      ((mp2-mp1)*(mp2+mp1))
        endif
        if (mn.ne.mneu) then
          MFphii=mneu**2/mn**2*MFphii
        endif
c... Convert from dPhi / dz dTheta to dPhi / dE dTheta
        MFphii=MFphii/mneu
      endif

      
 5000 format(' ',a,f8.2,a)
 5010 format(' ',a,i2,a,f8.2,a) 
 5020 format(' ',a,f8.2,a,a)

      return

      end
