      program ann2flx
      implicit none
c_______________________________________________________________________
c
c  Program to find how limits derived with a given energy
c  threshold converts to limits of a given other energy threshold.
c  Date: 99-09-08
c  Modified: 2000-06-14
c
c=======================================================================
c------------------------------------------------------------ Functions

      real*8 MFphi
      real murang

c------------------------------------------------------------ Variables

      real*8 mneu,thmax,flx,arate
      real*8 eth0,phi0,eth1,phi1,cf,phi0noth,phi1noth,denscorr
      integer ch,flxk,istat,i,flxk0,flxk1
      character*2 wh
      character*4 type(6)
      character*80 filename
      data type/'soft','soft','soft','hard','hard','hard'/
      data denscorr/0.92d0/  ! correction for sims at 1 g/cm^3

c---------------------------------------------------------------- Setup

c...Chose type of distributions here
c...101 differential flux of neutrinos
c...102 differential neutrino to muon conversion rate in the ice
c...  - use for putting muons out in a volume
c...103 differential flux of muons in the ice
c...  - use for putting muons out on the surface of the detector
c...  - includes energy loss and multiple Coulomb scattering
c      flxk=3 ! Integrated flux above a given threshold and
c             ! within a given angular window


c      do i=0,1000
c        eth0=dble(i)*dble(250.0d0)/dble(1000.0d0)
c        phi0 = MFphi(250.0d0,eth0,29.99d0,5,'ea',3,istat)
c        write(*,*) eth0,phi0
c      enddo
c      stop

c----------------------------------------------------------------------
      write(*,*)
      write(*,*) '                 ANN2FLX 1.02'
      write(*,*) '       (based on DarkSusy, version 3.07)'
      write(*,*) ' '
      write(*,*) '         A program to convert from an'
      write(*,*) '          annihilation rate to a flux'
      write(*,*) '    from WIMP annihilation in the Earth/Sun.'
      write(*,*) ' '
      write(*,*) '                June 14, 2000'
      write(*,*) '                   J. Edsjo'   
      write(*,*) '                edsjo@physto.se'
      write(*,*) ' '

      write(*,*) ' '
      write(*,*) 'Enter filename for results (''none'' for no file)'
      read(*,'(A)') filename
      write(*,*) filename

      if (filename.ne.'none') then
        open(unit=15,file=filename,status='new',form='formatted')
        write(15,96) 
        write(15,97) 
        write(15,98) 
        write(15,99)
 96     format('# ann2flx, v1.02, June 14, 2000. edsjo@physto.se')
 97     format('#',' AnnRate (s^-1) * ConvFact -> Flux (km^-2 yr^-1)',
     &   ' (fl=1 or 3)')
 98     format('#','  or nu to mu conversion rate (km^-3 yr^-1) (fl=2)')
 99     format('#','...mx...',1x,'wh',1x,
     &    'ch',1x,'type',1x,'.thmax.',1x,'..eth0..',1x,'fl',1x,
     &    '..ConvFact..',1x,'....arate...',1x,'.....flx....')
      endif

 10   write(*,*)
      write(*,*) 'Enter Neutralino mass [GeV] (0 to quit):'
      read(*,*) mneu
      write(*,*) mneu
      if (mneu.eq.0.0d0) then
        close(15)
        stop
      endif

      write(*,*) 'Annihilation in the Earth(''ea'') or the Sun(''su''):'
      read(*,'(a)') wh
      write(*,*) wh

      write(*,*) 'Annihilation channel?'
      write(*,*) '  1 = c c-bar'
      write(*,*) '  2 = b b-bar'
      write(*,*) '  3 = t t-bar'
      write(*,*) '  4 = tau^+ tau^-'
      write(*,*) '  5 = W^+ W^-'
      write(*,*) '  6 = Z^0 Z^0'
      read(*,*) ch
      write(*,*) ch


      write(*,*) 'Enter type of flux or conversion rate',
     &  '  you want to convert to'
      write(*,*) '  1 = neutrino flux (km^-2 yr^-1)'
      write(*,*) '  2 = neutrino to muon conversion rate (km^-3 yr^-1)'
      write(*,*) '  3 = muon flux (km^-2 yr^-1)'
      read(*,*) flxk0
      write(*,*) flxk0

      write(*,*) 'Enter energy threshold:'
      read(*,*) eth0
      write(*,*) eth0

      write(*,*) 'Half-aperture angle, theta:'
      write(*,*) '  <  30.00 -> actual angle'
      write(*,*) '  >= 30.00 -> all angles included'
      read(*,*) thmax
      if (thmax.gt.30.0d0) thmax=30.0d0
      write(*,*) thmax
     
      phi0 = MFphi(mneu,eth0,thmax,ch,wh,flxk0,istat)

      cf=3.16d13                ! m^-2 s^-1 -> km^-2 yr^-1
     &  *phi0                   ! MFphi
     &  *1.0d-30                ! factor in MFphi

      if (flxk0.eq.2) then
         cf=cf*1000.0d0         ! Another m^-1 -> km^-1 for the conv. rate
         cf=cf*denscorr         ! tables for dens = 1 g/cm^3
      endif

      write(*,*) 'Enter annihilation rate (s^-1)'
      read(*,*) arate
      write(*,*) arate

      flx=arate*cf

      if (filename.ne.'none') then
        write(15,100) mneu,wh,ch,type(ch),
     &    thmax,eth0,flxk0,cf,arate,flx
 100    format(1x,F8.2,1x,A2,1x,I2,1x,A4,1x,F7.2,1x,F8.2,1x,I2,1x,
     &    E12.6,1x,E12.6,1x,E12.6)
      endif

      write(*,*) ' '
      write(*,*) 'RESULTS'
      write(*,*) '======='
      write(*,*) ' '
      write(*,*) '  mneu  = ',mneu
      write(*,*) '  wh    = ',wh
      write(*,*) '  ch    = ',ch
      write(*,*) '  eth0  = ',eth0
      write(*,*) '  type0 = ',flxk0
      write(*,*) '  thmax = ',thmax
      write(*,*) '     ====> Conversion factor = ',cf
      write(*,*) 'Multiply your annihilation rate in s^-1 with'
      write(*,*) 'the conversion factor to get a limit on the'
      if (flxk0.eq.1) then
        write(*,*) 'neutrino flux in km^-2 yr^-1.'
      elseif (flxk0.eq.2) then
        write(*,*) 'neutrino to muon conversion rate in km^-3 yr^-1.'
      elseif (flxk0.eq.3) then
        write(*,*) 'muon flux in km^-2 yr^-1.'
      endif
      write(*,*) '  Annihilation rate (s^-1): ',arate
      if (flxk0.eq.1) then
        write(*,*) '  Neutrino flux (km^-2 yr^-1): ',flx
      elseif (flxk0.eq.2) then
        write(*,*) '  Neutrino to muon conversion rate (km^-3 yr^-1): ',
     &    flx
      elseif (flxk0.eq.3) then
        write(*,*) '  Muon flux (km^-2 yr^-1): ',flx
      endif

      goto 10 

      end

