      program degfind
      implicit none
c_______________________________________________________________________
c
c  Program to find which angular cut that corresponds to keeping
c  a given percentage of the signal
c  Author: Joakim Edsjo (edsjo@physto.se)
c  Date: 99-08-21
c
c=======================================================================
c------------------------------------------------------------ Functions

      real*8 MFphi
      real murang

c------------------------------------------------------------ Variables

      real*8 mneu,emuth,thmax,flux,e,t,phi0,eran,tran,rad,
     &  tmin,tmax,emin,emax,xran,yran,zran,fran,fkeep,eth
      real*8 thkeep,theta,phi,phikeep,rdif
      integer ch,flxk,istat,ngood,ntotal,nmuon,iseed
      integer ie,it,nt,i
      character*2 wh
      character*80 filename
      real ptot,px,py,pz,fi,rnunif,dummy
      parameter (rad = 57.2958d0)    

c---------------------------------------------------------------- Setup

c...Chose type of distributions here
c...101 differential flux of neutrinos
c...102 differentiel source function of muons in the ice
c...  - use for putting muons out in a volume
c...103 differential flux of muons in the ice
c...  - use for putting muons out on the surface of the detector
c...  - includes energy loss and multiple Coulomb scattering
      flxk=3 ! Integrated flux above a given threshold and
             ! within a given angular window

c----------------------------------------------------------------------
      write(*,*)
      write(*,*) '                 DEGFIND 1.00'
      write(*,*) '       (based on DarkSusy, version 3.07)'
      write(*,*) ' '
      write(*,*) '       A program to generate muon events'
      write(*,*) '    from WIMP annihilation in the Earth/Sun.'
      write(*,*) ' '
      write(*,*) '               August 21, 1999'
      write(*,*) '                   J. Edsjo'   
      write(*,*) '                edsjo@physto.se'
      write(*,*) ' '

      write(*,*)
      write(*,*) 'Enter Neutralino mass [GeV] (0 to quit):'
      read(*,*) mneu
      write(*,*) mneu
      if (mneu.eq.0.0d0) stop

      write(*,*) 'Annihilation in the Earth(''ea'') or the Sun(''su''):'
      read(*,'(a)') wh
      write(*,*) wh

      write(*,*) 'Annihilation channel?'
      write(*,*) '  1 = c c-bar'
      write(*,*) '  2 = b b-bar'
      write(*,*) '  3 = t t-bar'
      write(*,*) '  4 = tau^+ tau^-'
      write(*,*) '  5 = W^+ W^-'
      write(*,*) '  6 = Z^0 Z^0'
      read(*,*) ch
      write(*,*) ch

      write(*,*) 'Enter energy threshold:'
      read(*,*) eth
      write(*,*) eth

      write(*,*) 'Enter fraction to keep:'
      read(*,*) fkeep
      write(*,*) fkeep

      write(*,*) 'Now searching for an angular cut that keeps'
      write(*,*) 'the fraction ',fkeep,' of the signal above'
      write(*,*) eth,' GeV.'
      thmax=29.99d0
      phi0 = MFphi(mneu,eth,thmax,ch,wh,flxk,istat)
      nt=600
      rdif=10.0d0
      do i=1,nt-1
        theta=dble(i)/dble(nt)*30.0d0
        phi=MFphi(mneu,eth,theta,ch,wh,flxk,istat)
        if (abs(phi/phi0-fkeep).lt.rdif) then  ! better theta found
          thkeep=theta
          phikeep=phi
          rdif=abs(phi/phi0-fkeep)
        endif
      enddo


      write(*,*) 'Best theta found at'
      write(*,*) '  Theta  = ',thkeep,' degrees'
      write(*,*)   '  Flux at this theta: ',phikeep
      write(*,*)   '  Flux at max theta:  ',phi0
      write(*,*)   '  ==>  Fraction = ',phikeep/phi0
      end

