      program degopt
      implicit none
c_______________________________________________________________________
c
c  Program to find which angular cut that is optimal given
c  a limit as a function of theta.
c  Author: Joakim Edsjo (edsjo@physto.se)
c  Date: 99-08-21
c
c=======================================================================
c------------------------------------------------------------ Functions

      real*8 MFphi
      real murang

c------------------------------------------------------------ Variables

      real*8 mneu,emuth,thmax,flux,e,t,phi0,eran,tran,rad,
     &  tmin,tmax,emin,emax,xran,yran,zran,fran,fkeep,eth
      real*8 thkeep,theta,phi,phikeep,rdif,philimmin,pm
      real*8 phimacro,ethlim
      integer ch,flxk,istat,ngood,ntotal,nmuon,iseed
      integer ie,it,i,nt
      character*2 wh
      character*80 filename
      real ptot,px,py,pz,fi,rnunif,dummy
      parameter (rad = 57.2958d0)    

c---------------------------------------------------------------- Setup

c...Chose type of distributions here
c...101 differential flux of neutrinos
c...102 differentiel source function of muons in the ice
c...  - use for putting muons out in a volume
c...103 differential flux of muons in the ice
c...  - use for putting muons out on the surface of the detector
c...  - includes energy loss and multiple Coulomb scattering
      flxk=3 ! Integrated flux above a given threshold and
             ! within a given angular window

c----------------------------------------------------------------------
      write(*,*)
      write(*,*) '                 DEGOPT 1.00'
      write(*,*) '       (based on DarkSusy, version 3.07)'
      write(*,*) ' '
      write(*,*) '       A program to generate muon events'
      write(*,*) '    from WIMP annihilation in the Earth/Sun.'
      write(*,*) ' '
      write(*,*) '               August 21, 1999'
      write(*,*) '                   J. Edsjo'   
      write(*,*) '                edsjo@physto.se'
      write(*,*) ' '

 10   continue
      write(*,*)
      write(*,*) 'Enter Neutralino mass [GeV] (0 to quit):'
      read(*,*) mneu
      write(*,*) mneu
      if (mneu.eq.0.0d0) stop

      write(*,*) 'Annihilation in the Earth(''ea'') or the Sun(''su''):'
      read(*,'(a)') wh
      write(*,*) wh

      write(*,*) 'Annihilation channel?'
      write(*,*) '  1 = c c-bar'
      write(*,*) '  2 = b b-bar'
      write(*,*) '  3 = t t-bar'
      write(*,*) '  4 = tau^+ tau^-'
      write(*,*) '  5 = W^+ W^-'
      write(*,*) '  6 = Z^0 Z^0'
      read(*,*) ch
      write(*,*) ch

      write(*,*) 'Enter energy threshold of plot:'
      read(*,*) eth
      write(*,*) eth

      write(*,*) 'Enter energy threshold of limit (exp.):'
      read(*,*) ethlim
      write(*,*) ethlim

      thmax=29.99d0
      philimmin = 1.0d20
      phi0 = MFphi(mneu,eth,thmax,ch,wh,flxk,istat)

      nt=600
      do i=1,nt-1
        theta=dble(i)/dble(nt)*30.0d0
        phi=MFphi(mneu,ethlim,theta,ch,wh,flxk,istat)
        fkeep=phi/phi0
        pm=phimacro(wh,theta)/fkeep
        if (pm.lt.philimmin) then  ! better theta found
          thkeep=theta
          phikeep=phi
          philimmin=pm
        endif
      enddo


      write(*,*) ' ' 
      write(*,*) 'Best theta found at'
      write(*,*) '  Theta: ',thkeep,' degrees'
      write(*,*) '  Raw flux limit at this theta: ',
     &  philimmin*phikeep/phi0
      write(*,*) '  Fraction of signal kept: ',phikeep/phi0
      write(*,*) '  ==> Flux limit at this theta: ',philimmin

      goto 10

      end


***********************************************************************
*** PHIMACRO
***********************************************************************

      real*8 function phimacro(wh,th)
      implicit none
      character*2 wh
      real*8 theta(8,2),philim(8,2),thpl,th
      integer whi,thi,i

c...MACRO results, ICRC99
      data theta/0.0d0,3.0d0,6.0d0,9.0d0,15.0d0,18.0d0,24.0d0,30.0d0,
     &  0.0d0,3.0d0,6.0d0,9.0d0,15.0d0,18.0d0,24.0d0,30.0d0/
      data philim/1.35d-14,1.35d-14,1.07d-14,1.42d-14,2.07d-14,
     &   2.77d-14,4.06d-14,6.38d-14,
     &   2.89d-15,2.89d-15,5.07d-15,6.58d-15,1.03d-14,1.28d-14,
     &   1.56d-14,2.01d-14/

      if (wh.eq.'su'.or.wh.eq.'SU') then
        whi=1
      else
        whi=2
      endif

      thi=0
      do i=1,7
        if (theta(i,whi).le.th.and.theta(i+1,whi).gt.th) then
          thi=i
          goto 10
        endif
      enddo

 10   if (thi.eq.0) then
        phimacro=0.0d0
        return
      endif

      thpl=(th-theta(thi,whi))/(theta(thi+1,whi)-theta(thi,whi))
      phimacro=((1.0-thpl)*philim(thi,whi)+thpl*philim(thi+1,whi))
     &  *3.16d17  ! convert to km^-2 yr^-1
 
      return
      end


