      program wimpsimp
      implicit none
c_______________________________________________________________________
c
c  Program to generate event files from the muflux distributions.
c  Author: Joakim Edsjo (edsjo@physto.se)
c  Modified by: Ariel Goobar and Joakim Edsjo
c  Date: 99-02-04
c  Modified: 03-03-18 by Joakim Edsjo
c
c=======================================================================
c------------------------------------------------------------ Functions

      real*8 MFphi
      real murang

c------------------------------------------------------------ Variables

      real*8 mneu,emuth,thmax,flux,e,t,phimax,eran,tran,rad,
     &  tmin,tmax,emin,emax,xran,yran,zran,fran,wfun,weight
      integer ch,flxk,istat,ngood,ntotal,nmuon,iseed,wtype
      integer ie,it,ne,nt,jy,jm,jd
      character*2 wh
      character*15 progname
      character*20 progdate
      character*80 fnameold,fnamenew
      real ptot,px,py,pz,fi,rnunif,dummy
      parameter (rad = 57.2958d0)    
      parameter (progname='WimpSimp 1.2')
      parameter (progdate='March 18, 2003')
      character*1 muornu
      character*9 pname

c---------------------------------------------------------------- Setup

c...Chose type of distributions here
c...101 differential flux of neutrinos
c...102 differentiel source function of muons in the ice
c...  - use for putting muons out in a volume
c...103 differential flux of muons in the ice
c...  - use for putting muons out on the surface of the detector
c...  - includes energy loss and multiple Coulomb scattering
c      flxk=102 ! Differential source function of muons

c----------------------------------------------------------------------

      write(*,*)
      write(*,*) '                 ',progname
      write(*,*) '       (based on DarkSusy, version 3.07)'
      write(*,*) ' '
      write(*,*) '       A program to generate muon events'
      write(*,*) '    from WIMP annihilation in the Earth/Sun.'
      write(*,*) ' '
      write(*,*) '              ',progdate
      write(*,*) '                   J. Edsjo'   
      write(*,*) '                edsjo@physto.se'
      write(*,*) ' '

      write(*,*)
      write(*,*) 'Enter Neutralino mass [GeV] (0 to quit):'
      read(*,*) mneu
      write(*,*) mneu
      if (mneu.eq.0.0d0) stop

      write(*,*) 'Annihilation in the Earth(''ea'') or the Sun(''su''):'
      read(*,'(a)') wh
      write(*,*) wh

      write(*,*) 'Annihilation channel?'
      write(*,*) '  1 = c c-bar'
      write(*,*) '  2 = b b-bar'
      write(*,*) '  3 = t t-bar'
      write(*,*) '  4 = tau^+ tau^-'
      write(*,*) '  5 = W^+ W^-'
      write(*,*) '  6 = Z^0 Z^0'
      read(*,*) ch
      write(*,*) ch

      write(*,*) 'Muons or neutrinos? (m/n)'
      read(*,*) muornu
      write(*,*) muornu

      if (muornu.eq.'n'.or.muornu.eq.'N') then
        pname='neutrinos'
        flxk=101
      elseif (muornu.eq.'m'.or.muornu.eq.'M') then
        pname='muons'
        flxk=102
      else
        write(*,*) 'Invalid choice: ',muornu
        stop
      endif

      write(*,*) 'Do you want to add a weight function to',
     &  ' the distribution function?'
      write(*,*) ' 0: weight=1, i.e. no weighting'
      write(*,*) ' 1: weight=E, i.e. more events with high energy'
      write(*,*) ' 2: weight=E^2,',
     &  ' i.e. even more events with high energy'
      write(*,*) 'Note: These are the weight functions on the',
     &  ' distribution, the weights on the individual events',
     &  ' will be 1/weight.'
      read(*,*) wtype
      write(*,*) wtype

      write(*,*) 'Enter number of ',pname
      read(*,*) nmuon 
      write(*,*) nmuon

c      write(*,*) 'Enter output file (old HEMAS-like format):'
c      write(*,*) '(enter ''none'' if you do not want this output)'
c      read(*,'(a)') fnameold
c      write(*,*) fnameold
      fnameold='none'  ! Not using this anymore

      write(*,*) 'Enter output file (new format):'
      write(*,*) '(enter ''none'' if you do not want this output)'
      read(*,'(a)') fnamenew
      write(*,*) fnamenew


      write(*,*) 'Finding max of differential distribution...'
      phimax = 0.
      ne=500
      nt=600
      do ie=1,ne 
        do it=1,nt
          thmax = float(it)*30.0/float(nt)- 0.01
          emuth = 10. + (mneu-10.0)/float(ne) * float(ie-1)
          flux=MFphi(mneu,emuth,thmax,ch,wh,flxk,istat)
     &      *wfun(emuth,wtype)
          if (flux .gt. phimax) then 
            phimax = flux 
            e   = emuth
            t   = thmax
          endif
        enddo
      enddo
      write(*,*) 'Max found at:'
      write(*,500) '  Energy = ',e,' Gev'
      write(*,500) '  Theta  = ',t,' degrees'

C=== Event generation started

c- start generating muons with these fluxes

      call idate(jm,jd,jy)

      

      tmin =  0.
      tmax = 29.0

      if ((muornu.eq.'n'.or.muornu.eq.'N').and.
     &    (wh.eq.'su'.or.wh.eq.'SU')) then
        tmax=0.3d0
      endif
      emin = 10.
      emax = mneu 
      ngood = 0
      ntotal = 0
      call start_ran(iseed)

c      call rninit(497328) ! uncomment if you want a specific random seed

      if (fnameold.ne.'none') then
        open(unit=60,file=fnameold,form='formatted',status='unknown')
c      write(60,*) mneu,ch,nmuon
        write(60,*) nmuon
      endif

      if (fnamenew.ne.'none') then
        open(unit=61,file=fnamenew,form='formatted',status='unknown')
        write(61,'(''#'',A)') 'WF 1.1'
        write(61,'(''#'',A,A,A,A,A)') 'Creator: ',progname,
     &    '(version from ',progdate,')'
        write(61,111) 'Parameters: ',mneu,wh,ch,nmuon,wtype
        write(61,112) 'Date: ',jy,jm,jd
 111    format('#',A,1x,'mneu=',F9.2,1x,'where=',A2,1x,'channel=',I2,
     &    1x,'nevent=',I8,1x,'wtype=',I2)
 112    format('#',A,1x,i2,'-',i2,'-',i2)
      endif

 10   continue
* generate three uniform random numbers between in the cube
* (xmin:xmax,ymin:ymax,zmin:zmax)
*
*
        xran = rnunif(dummy)
        yran = rnunif(dummy)
        zran = rnunif(dummy)
*

        tran = (tmax-tmin)*xran + tmin
        eran = (emax-emin)*yran + emin
        fran = phimax*zran
        weight=wfun(eran,wtype)
        flux=MFphi(mneu,eran,tran,ch,wh,flxk,istat)*weight
        if (istat.ne.0) write(*,*) 'Something went wrong!'
        ntotal = ntotal + 1
        if (fran.le.flux) then
c-- keep the event!
          ngood = ngood + 1
          fi = rnunif(dummy)*2.0*3.14159
          px = eran*cos(fi)*sin(tran/rad)
          py = eran*sin(fi)*sin(tran/rad)
          pz = eran*cos(tran/rad)
          ptot = sqrt(px**2 + py**2 + pz**2)
          if (fnameold.ne.'none') then
            write(60,*) 1,0.0
            write(60,*) 0.0,0.0,0.0,0.0
            write(60,*) ptot,px,py,pz
            write(60,*) murang(ptot)
          endif

          if ((muornu.eq.'n'.or.muornu.eq.'N').and.
     &        (wh.eq.'su'.or.wh.eq.'SU')) then
            tran=0.0d0
          endif
 
          if (fnamenew.ne.'none') then
            if (muornu.eq.'n'.or.muornu.eq.'N') then
              write(61,121) ngood
              write(61,122) 'nu     ',tran,fi*180.0d0/3.14159d0,eran,
     &          1.0d0/weight
              write(61,123)
            else
              write(61,121) ngood
              write(61,122) 'mu     ',tran,fi*180.0d0/3.14159d0,eran,
     &          1.0d0/weight
              write(61,123)
            endif
          endif

 121      format('EVENT',1x,I8)
 122      format(A7,4(1x,e12.6))
 123      format('EE')

          if (ngood.ge.nmuon) goto 900
        endif

c        write(*,*) 'The differential muon flux at the given energy and',
c     &    ' angle is'
c        write(*,*) flux*1d-30,
c     &    ' GeV^-1 (degrees)^-1 m^-2 (annihilation)^-1.'
c        if (istat.ne.0) write(*,*) 'Something went wrong!'

      goto 10
900   continue
      if (fnameold.ne.'none') close(60)
      if (fnamenew.ne.'none') close(60)

      write(*,*) ' '
      write(*,*) 'SUMMARY'
      write(*,*) ' '
      write(*,500) 'Neutralino mass: ',mneu,' GeV'
 500  format(1x,A,f10.4,A)
      if (wh.eq.'ea'.or.wh.eq.'EA') then
        write(*,500) 'Annihilation in the Earth.'
      else
        write(*,500) 'Annihilation in the Sun.'
      endif
      write(*,'(1x,A,I2)') 'Annihilation channel: ',ch
      write(*,*) 'Filename (old format): ',fnameold
      write(*,*) 'Filename (new format): ',fnamenew
      write(*,*) 'Generated number: ',ntotal
      write(*,*) '   of which good: ',ngood
      write(6,'(1x,a,f12.10)') '   -> efficiency in generation: ',
     &  float(ngood)/float(ntotal)

      end


**********************************************************************

      real*8 function wfun(e,type)
      real*8 e
      integer type

      if (type.eq.0) then
        wfun=1.0d0
      elseif (type.eq.1) then
        wfun=e
      elseif (type.eq.2) then
        wfun=e**2
      else
        write(*,*) 'ERROR: Invalid weight type = ',type
        stop
      endif

      return
      end


**********************************************************************
**********************************************************************
**********************************************************************

      subroutine start_ran(iseed)
C INITILAIZE RANDOM NUMBER GENERATION
*
       call rnclck ( iseed )
       call rninit ( iseed )
      end 


      SUBROUTINE RNCLCK ( ISEED )
*
*            Clock start of random number generator
*
*     Calculate number of seconds from 1 January 1989 modulus 921350144
*     which may be used as start seed for the sequence. This implies
*     that one avoids using the same start sequence in any simulation
*     unless two jobs are ran in parallel during the same second!
*     The code previously used the CERN library routine DATIMH to
*     obtain the system date and time. This, however, is a unnecessary
*     detour in achieving this. A new subroutine RNDATE has been
*     supplied which directly uses system functions for date and time
*     on most computers.
*
*     Working in several laboratories one may, as has been done for
*     DELPHI, construct a value from a lab-code and the number of
*     seconds past during the current year.
*
*                                                    Ch.Walck 1989-12-21
*                                                     revised 1990-09-22
*                                                     bug fix 1994-09-05
*
      SAVE IMD
      DIMENSION IMD(12)
      DATA IMD / 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 /
*
*     Get system date and time
*
      CALL RNDATE ( JY, JM, JD, IH, IM, IS )
*
*     Calculate number of days in previous years from 1 January 1989
*
      IY = 1900 + JY
      IF ( JY .LT. 89 ) IY = 2000 + JY
      IDY = 0
      IF ( IY .GT. 1989 ) THEN
         DO 100 I = 1989, IY-1
            JDY = 365
*           Code valid until 2100 which is not a leap-year (2000 is! 940905)
            IF ( MOD(I,4).EQ.0 ) JDY = 366
            IDY = IDY + JDY
  100    CONTINUE
      END IF
*
*     Number of days from 1 January until 1st current month
*     (treat leap-years as well)
*
      LY =0
*     Code valid until 2100 which is not a leap-year (2000 is! 940905)
      IF ( MOD(JY,4).EQ.0 ) LY = 1
      ISHIFT = IMD(JM)
      IF ( JM .GT. 2 ) ISHIFT = IMD(JM) + LY
*
*     Seconds since January 1st 1989 at 00:00:00 (=0)
*
      ISEED = IS + 60 * ( IM + 60 * ( IH + 24 * ( JD-1+ISHIFT+IDY ) ) )
*
*     Bring packed seed into range (this modulus will zero the ISEED at
*     18:35:44 on March 13 2018 and at 13:11:28 on May 24 2047, ... )
*
      ISEED = MOD(ISEED,921350144)
*
      WRITE (6,1003) JD, JM, JY, IH, IM, IS, ISEED
 1003 FORMAT (/' RNCLCK: date and time is',I3,'-',I2.2,'-',I2.2,I3.2,':'
     +       ,I2.2,':',I2.2,', clock start at seed',I10/)
      RETURN
      END

      SUBROUTINE RNDATE ( JY, JM, JD, IH, IM, IS )
*
*     Routine to give current date and time. This routine replaces the
*     old DATIMH routine from the CERN library which was an unnecessary
*     detour in achieving this information. The routine is not fully
*     tested on all computers but hopefully works anyway. If any problem
*     is encountered the self-tag DATIMH activates the old code.
*
*          JY - Year (modulus 100)        IH - Hours
*          JM - Month                     IM - minutes
*          JD - Date                      IS - seconds
*
*                                                    Ch.Walck 1990-09-22
*
*     System calls on VAX
*
      CHARACTER*8 HMS
      CALL IDATE ( JM, JD, JY )
      CALL TIME  ( HMS )
      READ (HMS,1001) IH, IM, IS
 1001 FORMAT (I2,1X,I2,1X,I2)
      RETURN
      END

      SUBROUTINE RNINIT ( ISEED )
*
*     Initialize random number generator by Marsaglia & Zaman (& Tsang).
*
*     The initial values for U(97) are generated by using a 3-lag
*     Fibonacci generator with 3 initial seeds and a congruential
*     generator with one initial seed.
*
*     - The values of the three initial seeds for the 3-lag Fibonacci
*       generator may be in the range 1 to 178. This generator has a
*       period of 32044 in about half of all cases. In about a quarter
*       of all cases each the period is 16022 and 8011. In a few cases
*       with combinations like 1, 1, 178 or 178, 1, 178 the period is
*       only 1, 2 or 4. The authors proposes to avoid the combination
*       1, 1, 1 but we choose to be somewhat more restrictive and use
*       only values from 2 to 177 for the three seeds.
*     - The initial seed for the congruential generator may be in the
*       range 0 to 168. The period of this generator is 169 regardless
*       of the initial seed value.
*
*     The four initial seeds are packed into one integer which can be
*     regarded as a four digit number with a variable base for each
*     digit. This in order to get a one-to-one correspondence between
*     the four seeds and the big integer without leaving any holes.
*     With I, J, K being the seeds of the Fibonacci generator and L the
*     seed of the congruential generator we pack them as:
*        ISEED = (I-2)*176*176*169 + (J-2)*176*169 + (K-2)*169 + L
*     ISEED can thus have the range 0 to 169*176**3-1=921350143 all
*     values giving a legal and unique I, J, K, L combination.
*
*                                                    Ch.Walck 1989-12-21
*
      COMMON /RNCMMN/ LAG1, LAG2, U(97), C, CD, CM
*
      LAG1 = 97
      LAG2 = 33
*
*     Abort run if ISEED value is out of its valid range
*
      IF ( ISEED.LT.0 .OR. ISEED.GT.921350143 ) THEN
         WRITE (6,1001) ISEED
 1001    FORMAT (//' RNINIT: error - ISEED out of range, ISEED=',I11
     +          ,'), run aborted.'//)
         STOP
      END IF
*
*     Unpack seeds for 3-lag Fibonacci and congruential generator
*
      I = MOD(ISEED/169/176/176,176) + 2
      J = MOD(ISEED/169/176,176) + 2
      K = MOD(ISEED/169,176) + 2
      L = MOD(ISEED,169)
*
*     Floating point arithmetics version
*
      DO 200 II = 1, 97
         S = 0.0
         T = 0.5
         DO 100 JJ = 1, 24
            M = MOD(MOD(I*J,179)*K,179)
            I = J
            J = K
            K = M
            L = MOD(53*L+1,169)
            IF ( MOD(L*M,64) .GE. 32 ) S = S + T
            T = 0.5 * T
  100    CONTINUE
         U(II) = S
  200 CONTINUE
      C  =   362436.0 / 16777216.0
      CD =  7654321.0 / 16777216.0
      CM = 16777213.0 / 16777216.0
      RETURN
      END

      FUNCTION RNUNIF ( DUM )
*
*     Universal pseudorandom number generator by George Marsaglia & Arif
*     Zaman as described in the report FSU-SCRI-87-50, Florida State
*     University, 1987 (later on published together with a third author,
*     Wai Wan Tsang, in Stat. & Prob. Lett. 9 (1990) 35-39).
*
*     Initialize by a call to RNINIT and make use of, if wanted, the
*     utilities to pack/unpack seeds or use a clock-start.
*
*                                                    Ch.Walck 1989-12-21
*
      COMMON /RNCMMN/ LAG1, LAG2, U(97), C, CD, CM
*
*     Floating point arithmetics version
*
    1 UNI = U(LAG1) - U(LAG2)
      IF ( UNI .LT. 0.0 ) UNI = UNI + 1.0
      U(LAG1) = UNI
      LAG1 = LAG1 - 1
      IF ( LAG1 .EQ. 0 ) LAG1 = 97
      LAG2 = LAG2 - 1
      IF ( LAG2 .EQ. 0 ) LAG2 = 97
      C = C - CD
      IF ( C .LT. 0.0 ) C = C + CM
      UNI = UNI - C
      IF ( UNI .LT. 0.0 ) UNI = UNI + 1.0
*     Avoid returning zero
      IF ( UNI .EQ. 0.0 ) GO TO 1
      RNUNIF = UNI
      RETURN
      END

c      SUBROUTINE TIMEX ( T )
c      INCLUDE '($JPIDEF)'
c      CHARACTER*20 STRING
c      ISTAT = LIB$GETJPI ( JPI$_CPUTIM, , , ITIME, STRING, LEN )
c      IF ( .NOT.ISTAT ) CALL LIB$SIGNAL (%VAL(ISTAT))
c      T = FLOAT(ITIME) / 100.
c      RETURN
c      END

      FUNCTION RNNRMT ( XMU, XSIG )
C-----------------------------------------------------------------------
C
C...       FUNCTION RNNRMT ( XMU, XSIG )                    Ch.Walck    861005
C.
C.    Gaussian random number with mean XMU and standard deviation XSIG.
C.
C.    F(X;XMU,XSIG) = EXP(-((X-XMU)/XSIG)**2/2) / SQRT(2*PI) / XSIG
C.
C.    Variable:          X        (real)
C.    Parameters:        XMU      (real)
C.                       XSIG > 0 (real)
C.
C.    Expectation value: E(Z) = XMU
C.    Variance:          V(Z) = XSIG**2
C.    Skewness:         G1(Z) = 0
C.    Kurtosis:         G2(Z) = 0
C.
C.    Technique:
C.    Trapezoidal method of Ahrens (1972) as given in the article
C.    "Computer Methods for Sampling From the Exponential and Normal
C.    Distributions" by J.H.Ahrens and U.Dieter in Communications of
C.    the ACM Vol.15 (1972) 873. In fact this is the fastest of the
C.    routines given here! (However, the CERN routine NORRAN, coded
C.    in assembler language, is faster.)
C.
C.----------------------------------------------------------------------
      PARAMETER (   A=0.919544405706926,    A1=0.954057274493068,
     +             A2=0.978195649518392,    A3=0.993692071999222,
     +             X1=0.2897295736,         X2=1.84039874739771,
     +             X3=2.11402808333742,   X3SQ=4.46911473713927,
     +           DX12=1.55066917379771,   DX23=0.273629335939706,
     +           XINT=0.443299125820220, SLOPE=0.209694057195486,
     +            YMX=0.398942280401433,    YT=0.382544556042518,
     +             Y1=0.016397724358915,    Y2=0.015974522655238,
     +             Y3=0.042702581590795,    Q1=2.40375765693742,
     +             Q2=1.983915619969365)
C
      U  = RNUNIF(DUM)
      U0 = RNUNIF(DUM)
C
C     From maximal trapezoid inscribed in N(0,1)
C    -2.11 < X < 2.11 with P = 91.95 %
C
      IF ( U .LT. A ) THEN
         X = Q1*U0 + Q2*U - X3
         GO TO 100
C
C     From tail of N(0,1)
C     X > 2.11 with P = 3.45 % (sign decided below)
C
      ELSE IF ( U .LT. A1 ) THEN
    1    X  = X3SQ-2.0*LOG(RNUNIF(DUM))
         IF ( X*RNUNIF(DUM)**2 .GT. X3SQ ) GO TO 1
         X = SQRT(X)
C
C     From 2nd region between N(0,1) and trapezoid
C     0.29 < X < 1.84 with P = 2.41 % (sign decided below)
C
      ELSE IF ( U .LT. A2 ) THEN
    2    X = X1 + DX12 * RNUNIF(DUM)
         IF ( YMX*EXP(-X**2/2.0)-XINT+X*SLOPE.LT.Y2*RNUNIF(DUM)) GO TO 2
C
C     From 3rd region between N(0,1) and trapezoid
C     1.84 < X < 2.11 with P = 1.55 % (sign decided below)
C
      ELSE IF ( U .LT. A3 ) THEN
    3    X = X2 + DX23 * RNUNIF(DUM)
         IF ( YMX*EXP(-X**2/2.0)-XINT+X*SLOPE.LT.Y3*RNUNIF(DUM)) GO TO 3
C
C     From 1st (top) region between N(0,1) and trapezoid
C     0 < X < 0.29 with P = 0.63 % (sign decided below)
C
      ELSE
    4    X = X1 * RNUNIF(DUM)
         IF ( YMX*EXP(-X**2/2.0)-YT .LT. Y1*RNUNIF(DUM) ) GO TO 4
      END IF
C
C     Attach sign
C
      IF ( U0 .GE. 0.5 ) X = -X
C
  100 RNNRMT = X*XSIG + XMU
      RETURN
      END







